@@ -1,8 +1,9 @@
-import cdsapi
 import os
 
-def test_request():
+import cdsapi
 
+
+def test_request():
     c = cdsapi.Client()
 
     r = c.retrieve(
@@ -11,7 +12,7 @@ def test_request():
             "variable": "2t",
             "product_type": "reanalysis",
             "date": "2012-12-01",
-            "time": "12:00"
+            "time": "12:00",
         },
     )
 
